@@ -5,9 +5,6 @@ import html
 import json
 import base64
 from collections import OrderedDict
-import urllib.request, urllib.error, urllib.parse
-
-from utils.dateutils import DateUtils
 
 from .webtools import (
     calculate_hash,
@@ -15,8 +12,12 @@ from .webtools import (
     json_encode_field,
     json_decode_field,
     date_str_to_date,
+    WebLogger,
 )
+from .contentinterface import ContentInterface
+from .pages import PageFactory
 from .statuses import *
+from .utils.dateutils import DateUtils
 
 
 class ResponseHeaders(object):
@@ -139,6 +140,7 @@ class ResponseHeaders(object):
             return self.headers["Location"]
 
 
+
 class PageResponseObject(object):
     STATUS_CODE_OK = 200
     STATUS_CODE_ERROR = 500
@@ -207,16 +209,17 @@ class PageResponseObject(object):
                 self.text = self.binary.decode(self.encoding)
             except Exception as E:
                 WebLogger.exc(
-                    E, "Cannot properly decode ansower from {}".format(self.url)
+                    E, "Cannot properly decode text from {}".format(self.url)
                 )
                 self.text = self.binary.decode(self.encoding, errors="ignore")
+                self.add_error("Cannot properly decode text from {}".format(self.url))
 
         if self.text and not self.binary:
             try:
                 self.binary = self.text.encode(self.encoding)
             except Exception as E:
                 WebLogger.exc(
-                    E, "Cannot properly encode ansower from {}".format(self.url)
+                    E, "Cannot properly encode text from {}".format(self.url)
                 )
                 self.binary = self.text.encode(self.encoding, errors="ignore")
 
@@ -488,79 +491,17 @@ class PageResponseObject(object):
         if binary:
             return calculate_hash_binary(binary)
 
+    def is_captcha_protected(self):
+        interface = ContentInterface(url=self.url, contents = self.get_text())
+        return interface.is_captcha_protected()
 
-def get_response_to_bytes(response):
-    from .ipc import string_to_command
-
-    total_bytes = bytearray()
-
-    bytes1 = string_to_command("PageResponseObject.__init__", "OK")
-    bytes2 = string_to_command("PageResponseObject.url", response.url)
-    bytes3 = string_to_command(
-        "PageResponseObject.status_code", str(response.status_code)
-    )
-    if response.headers != None:
-        bytes4 = string_to_command(
-            "PageResponseObject.headers", json.dumps(response.headers)
-        )
-    else:
-        bytes4 = bytearray()
-
-    if response.text:
-        bytes5 = string_to_command("PageResponseObject.text", response.text)
-    elif response.binary:
-        bytes5 = bytes_to_command("PageResponseObject.text", response.binary)
-    else:
-        bytes5 = bytearray()
-    bytes6 = string_to_command(
-        "PageResponseObject.request_url", str(response.request_url)
-    )
-    bytes7 = string_to_command("PageResponseObject.__del__", "OK")
-
-    total_bytes.extend(bytes1)
-    total_bytes.extend(bytes2)
-    total_bytes.extend(bytes3)
-    total_bytes.extend(bytes4)
-    total_bytes.extend(bytes5)
-    total_bytes.extend(bytes6)
-    total_bytes.extend(bytes7)
-
-    return total_bytes
-
-
-def get_response_from_bytes(all_bytes):
-    from .ipc import commands_from_bytes
-
-    response = PageResponseObject("")
-
-    commands_data = commands_from_bytes(all_bytes)
-    for command_data in commands_data:
-        if command_data[0] == "PageResponseObject.__init__":
-            pass
-        elif command_data[0] == "PageResponseObject.url":
-            response.url = command_data[1].decode()
-        elif command_data[0] == "PageResponseObject.request_url":
-            response.request_url = command_data[1].decode()
-        elif command_data[0] == "PageResponseObject.headers":
-            try:
-                response.headers = json.loads(command_data[1].decode())
-            except ValueError as E:
-                WebLogger.exc(E, "Exception when loading headers")
-        elif command_data[0] == "PageResponseObject.status_code":
-            try:
-                response.status_code = int(command_data[1])
-            except ValueError as E:
-                WebLogger.exc(E, "Exception when loading headers")
-        elif command_data[0] == "PageResponseObject.text":
-            response.set_text(command_data[1].decode())
-        elif command_data[0] == "PageResponseObject.__del__":
-            pass
-
-    return response
+    def get_page(self):
+        return PageFactory.get(self, self.get_text())
 
 
 def response_to_json(response, with_streams=False):
-    """ """
+    """
+    """
     response_data = OrderedDict()
 
     if response:
@@ -579,9 +520,7 @@ def response_to_json(response, with_streams=False):
 
         response_data["crawl_time_s"] = response.crawl_time_s
         response_data["Content-Type"] = response.get_content_type()
-        response_data["Recognized-Content-Type"] = (
-            response.get_recognized_content_type()
-        )
+        response_data["Recognized-Content-Type"] = response.get_recognized_content_type()
         response_data["Content-Length"] = response.get_content_length()
         response_data["Last-Modified"] = response.get_last_modified()
         response_data["Charset"] = response.get_encoding()
@@ -601,16 +540,15 @@ def response_to_json(response, with_streams=False):
             for error in response.errors:
                 response_data["errors"].append(error)
 
+        response_data["text"] = response.get_text()
+        response_data["binary"] = json_encode_field(response.get_binary())
+
         if with_streams:
             response_data["streams"] = response.get_streams()
-            response_data["text"] = response.get_text()
-            response_data["binary"] = json_encode_field(response.get_binary())
     else:
         response_data["is_valid"] = False
         response_data["status_code"] = HTTP_STATUS_CODE_EXCEPTION
-        response_data["status_code_str"] = status_code_to_text(
-            HTTP_STATUS_CODE_EXCEPTION
-        )
+        response_data["status_code_str"] = status_code_to_text(HTTP_STATUS_CODE_EXCEPTION)
 
     return response_data
 
@@ -644,3 +582,22 @@ def json_to_response(json_data, with_streams=False):
     response.body_hash = body_hash
 
     return response
+
+
+def response_to_file(response, file_name):
+    if not response:
+        return
+
+    with open(file_name, "w") as fh:
+        json_data = response_to_json(response)
+        json_text = json.dumps(json_data)
+        
+        fh.write(json_text)
+
+
+def file_to_response(file_name):
+    with open(file_name, "r") as fh:
+        json_text = fh.read()
+        json_data = json.loads(json_text)
+
+        return json_to_response(json_data)
